@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1995-2020, University of Amsterdam
+    Copyright (c)  1995-2021, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -43,6 +44,8 @@
 :- use_module(library(option)).
 :- use_module(library(error)).
 :- use_module(library(apply)).
+:- autoload(library(shlib), [current_foreign_library/2]).
+:- autoload(library(prolog_autoload), [autoload_all/1]).
 
 /** <module> Save current program as a state or executable
 
@@ -77,7 +80,7 @@ save_option(pce,         boolean,
             "Do (not) include the xpce graphics subsystem").
 save_option(packs,       boolean,
             "Do (not) attach packs").
-save_option(class,       oneof([runtime,development]),
+save_option(class,       oneof([runtime,development,prolog]),
             "Development state").
 save_option(op,          oneof([save,standard]),
             "Save operators").
@@ -674,14 +677,16 @@ save_predicate(P, SaveClass) :-
     feedback('~nsaving ~w/~d ', [F, A]),
     (   (   H = resource(_,_)
         ;   H = resource(_,_,_)
-        ),
-        SaveClass \== development
-    ->  save_attribute(P, (dynamic)),
-        (   M == user
-        ->  save_attribute(P, (multifile))
-        ),
-        feedback('(Skipped clauses)', []),
-        fail
+        )
+    ->  (   SaveClass == development
+        ->  true
+        ;   save_attribute(P, (dynamic)),
+            (   M == user
+            ->  save_attribute(P, (multifile))
+            ),
+            feedback('(Skipped clauses)', []),
+            fail
+        )
     ;   true
     ),
     (   no_save(P)
@@ -1004,7 +1009,7 @@ strip_file(File, Stripped) :-
 strip_file(File, File).
 
 do_strip_file(Strip, File, Stripped) :-
-    format(atom(Cmd), '"~w" -o "~w" "~w"',
+    format(atom(Cmd), '"~w" -x -o "~w" "~w"',
            [Strip, Stripped, File]),
     shell(Cmd),
     exists_file(Stripped).
