@@ -34,7 +34,8 @@
 :- module(mqi,
           [ mqi_start/0,
             mqi_start/1,                % +Options
-            mqi_stop/1                  % ?Thread
+            mqi_stop/1,                 % ?Thread
+            mqi_version/2               % ?Major_Version, ?Minor_Version
           ]).
 
 /**
@@ -109,6 +110,7 @@ Redirects STDOUT and STDERR to the file path specified.  Useful for debugging th
 :- use_module(library(socket)).
 :- use_module(library(http/json)).
 :- use_module(library(http/json_convert)).
+:- use_module(library(http/http_stream)).
 :- use_module(library(option)).
 :- use_module(library(term_to_json)).
 :- use_module(library(debug)).
@@ -137,6 +139,40 @@ Redirects STDOUT and STDERR to the file path specified.  Useful for debugging th
 % that can be cancelled
 :- dynamic(safe_to_cancel/1).
 
+%!  mqi_version(?Major_Version, ?Minor_Version) is det.
+%
+%  Provides the major and minor version number of the protocol used by the MQI.
+%  The protocol includes the message format and the messages that can
+%  be sent and received from the MQI.
+%
+%  Note that the initial version of the MQI did not have a version predicate so
+%  The proper way for callers to check the version is:
+%
+%  use_module(library(mqi)),
+%  (   current_predicate(mqi_version/2)
+%  ->  mqi_version(Major_Version, Minor_Version)
+%  ;   Major_Version = 0, Minor_Version = 0
+%  )
+%
+%  Major versions are increased when there is a change to the protocol that will
+%  likely break clients written to the previous version. Minor versions are increased
+%  when there is new functionality that will *not* break clients written to the old version
+%
+%  This allows a client written to MQI version 'Client_Major_Version.Client_Minor_Version'
+%  to check for non-breaking compatibility like this:
+%
+%  Client_Major_Version = MQI_Major_Version and Client_Minor_Version <= MQI_Minor_Version
+%
+%  Breaking changes (i.e. Major version increments) should be very rare as the goal is to
+%  have the broadest adoption possible.
+%
+%  Protocol Version History:
+%  0.0 ->   First published version. Had a protocol bug that required messages sent to MQI to
+%           count Unicode code points instead of bytes for the message header.
+%  1.0 ->   Breaking change: Fixed protocol bug so that it properly accepted byte count instead of Unicode code point
+%           count in the message header for messages sent to MQI.
+mqi_version(1, 0).
+
 
 % Password is carefully constructed to be a string (not an atom) so that it is not
 % globally visible
@@ -197,7 +233,7 @@ opt_type(write_output_to_file,      write_output_to_file,      file(write)).
 
 opt_help(port,                      "TCP/IP port for clients to connect to").
 opt_help(create_unix_domain_socket, "Create a Unix domain socket for clients to connect to").
-opt_help(unix_domain_socket,        "File path for the Unix domin socket").
+opt_help(unix_domain_socket,        "File path for the Unix domain socket").
 opt_help(password,                  "Connection password").
 opt_help(pending_connections,       "Max number of queued connections (5)").
 opt_help(query_timeout,             "Max query runtime in seconds (default infinite)").
@@ -412,12 +448,12 @@ create_connection(Server_Thread_ID, AcceptFd, Password, Encoding, Query_Timeout,
 goal_thread(Respond_To_Thread_ID) :-
     thread_self(Self_ID),
     throw_if_testing(Self_ID),
-    thread_get_message(Self_ID, goal(Goal, Binding_List, Query_Timeout, Find_All)),
-    debug(mqi(query), "Received Findall = ~w, Query_Timeout = ~w, binding list: ~w, goal: ~w", [Find_All, Query_Timeout, Binding_List, Goal]),
+    thread_get_message(Self_ID, goal(Unexpanded_Goal, Binding_List, Query_Timeout, Find_All)),
+    expand_goal(Unexpanded_Goal, Goal),
+    debug(mqi(query), "Received Findall = ~w, Query_Timeout = ~w, binding list: ~w, unexpanded: ~w, goal: ~w", [Find_All, Query_Timeout, Binding_List, Unexpanded_Goal, Goal]),
     (   Find_All
     ->  One_Answer_Goal = findall(Binding_List, @(user:Goal, user), Answers)
-    ;
-        One_Answer_Goal = ( @(user:Goal, user),
+    ;   One_Answer_Goal = ( @(user:Goal, user),
                             Answers = [Binding_List],
                             send_next_result(Respond_To_Thread_ID, Answers, _, Find_All)
                           )
@@ -427,17 +463,12 @@ goal_thread(Respond_To_Thread_ID) :-
     ->  catch(All_Answers_Goal, Top_Exception, true)
     ;   catch(call_with_time_limit(Query_Timeout, All_Answers_Goal), Top_Exception, true)
     ),
-    (
-        var(Top_Exception)
-    ->  (
-            Find_All
-        ->
-            send_next_result(Respond_To_Thread_ID, Find_All_Answers, _, Find_All)
-        ;
-            send_next_result(Respond_To_Thread_ID, [], no_more_results, Find_All)
+    (   var(Top_Exception)
+    ->  (   Find_All
+        ->  send_next_result(Respond_To_Thread_ID, Find_All_Answers, _, Find_All)
+        ;   send_next_result(Respond_To_Thread_ID, [], no_more_results, Find_All)
         )
-    ;
-        send_next_result(Respond_To_Thread_ID, [], Top_Exception, true)
+    ;   send_next_result(Respond_To_Thread_ID, [], Top_Exception, true)
     ),
     goal_thread(Respond_To_Thread_ID).
 
@@ -513,7 +544,8 @@ communication_thread_listen(Password, Socket, Encoding, Server_Thread_ID, Goal_T
     (   Password == Sent_Password
     ->  (   debug(mqi(protocol), "Password matched.", []),
             thread_self(Self_ID),
-            reply(Write_Stream, true([[threads(Self_ID, Goal_Thread_ID)]]))
+            mqi_version(Major, Minor),
+            reply(Write_Stream, true([[threads(Self_ID, Goal_Thread_ID), version(Major, Minor)]]))
         )
     ;   (   debug(mqi(protocol), "Password mismatch, failing. ~w", [Sent_Password]),
             reply_error(Write_Stream, password_mismatch),
@@ -846,6 +878,7 @@ reply(Stream, Term) :-
 % Special handling for exceptions since they can have parts that are not
 % "serializable". Ensures they they are always returned in an exception/1 term
 reply_error(Stream, Error_Term) :-
+    debug(mqi(query), "Responding with exception: ~w", [Error_Term]),
     (   error(Error_Value, _) = Error_Term
     ->  Response = exception(Error_Value)
     ;   (   atom(Error_Term)
@@ -878,7 +911,13 @@ write_message(Stream, String) :-
 % in whatever encoding is being used
 read_message(Stream, String) :-
     read_string_length(Stream, Length),
-    read_string(Stream, Length, String).
+    stream_property(Stream, encoding(Encoding)),
+    setup_call_cleanup(
+         stream_range_open(Stream, Tmp, [size(Length)]),
+         ( set_stream(Tmp, encoding(Encoding)),
+           read_string(Tmp, _, String)
+         ),
+         close(Tmp)).
 
 
 % Terminate with '.\n' so we know that's the end of the count
