@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2021, University of Amsterdam
+    Copyright (c)  2011-2022, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -1806,7 +1806,7 @@ Sseek64(IOSTREAM *s, int64_t pos, int whence)
 
   if ( s->functions->seek64 )
     pos = (*s->functions->seek64)(s->handle, pos, whence);
-  else if ( pos <= LONG_MAX )
+  else if ( s->functions->seek && pos <= LONG_MAX && pos >= LONG_MIN )
     pos = (*s->functions->seek)(s->handle, (long)pos, whence);
   else
   { errno = EINVAL;
@@ -1924,8 +1924,8 @@ object, which in turn calls the  ->unlink   which  may wish to close the
 associated stream.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-int
-Sclose(IOSTREAM *s)
+static int
+S__close(IOSTREAM *s, int flags)
 { int rval = 0;
 
   if ( s->magic != SIO_MAGIC )		/* already closed!? */
@@ -1943,7 +1943,19 @@ Sclose(IOSTREAM *s)
     return -1;
   }
 
-  SLOCK(s);
+  if ( (flags & (SIO_CLOSE_TRYLOCK|SIO_CLOSE_FORCE)) )
+  { if ( !STRYLOCK(s) )
+    { if ( (flags&SIO_CLOSE_FORCE) )
+      { PL_free(s->mutex);
+	s->mutex = NULL;
+      } else
+      { errno = EDEADLK;
+	return -1;
+      }
+    }
+  } else
+    SLOCK(s);
+
   s->flags |= SIO_CLOSING;
   rval = S__removebuf(s);
   if ( s->mbstate )
@@ -1988,6 +2000,16 @@ Sclose(IOSTREAM *s)
   return rval;
 }
 
+int
+Sclose(IOSTREAM *s)
+{ return S__close(s, 0);
+}
+
+int
+Sgcclose(IOSTREAM *s, int flags)
+{ return S__close(s, flags);
+}
+
 
 		 /*******************************
 		 *	     STRING I/O		*
@@ -2116,6 +2138,24 @@ next_chr(const char **s, IOENC enc)
 #define OUTCHR(s, c)	do { printed++; \
 			     if ( Sputcode((c), (s)) < 0 ) goto error; \
 			   } while(0)
+#define OUTFS() \
+	do \
+	{ if ( fs == fbuf ) \
+	  { while(fs < fe) \
+	    { int c = next_chr((const char**)&fs, enc); \
+	      OUTCHR(s, c); \
+	    } \
+	  } else \
+	  { for(;;) \
+	    { int c = next_chr((const char**)&fs, enc); \
+	      if ( c ) \
+		OUTCHR(s, c); \
+	      else \
+		break; \
+	    } \
+	  } \
+	} while(0)
+
 #define valdigit(c)	((c) - '0')
 #define A_LEFT	0			/* left-aligned field */
 #define A_RIGHT 1			/* right-aligned field */
@@ -2245,8 +2285,11 @@ Svfprintf(IOSTREAM *s, const char *fm, va_list args)
 
 	switch(*fm)
 	{ case 'c':
-	    *fe++ = va_arg(args, int);
+	  { int c = va_arg(args, int);
+	    fe = utf8_put_char(fe, c);
+	    enc = ENC_UTF8;
 	    break;
+	  }
 	  case 'p':
 	  { void *ptr = va_arg(args, void*);
 	    char fmbuf[8], *fp=fmbuf;
@@ -2353,17 +2396,20 @@ Svfprintf(IOSTREAM *s, const char *fm, va_list args)
 	    break;
 	}
 
+	/* Now `fs` is either the result of some sub-formatting or the
+           argument of `%s`.  In the latter case it is by definition
+	   0-terminated.  In the former case it may hold 0-bytes, e.g.,
+	   `%c` using a 0 argument.  `fe` points at the end.
+	*/
+
 	if ( has_arg1 )			/* aligned field */
 	{ if ( fs == fbuf )
 	    *fe = '\0';
 
 	  if ( align == A_LEFT )
-	  { int w = 0;
-	    while(*fs)
-	    { int c = next_chr((const char**)&fs, enc);
-	      OUTCHR(s, c);
-	      w++;
-	    }
+	  { int printed0 = printed;
+	    OUTFS();
+	    int w = printed-printed0;
 	    while(w < arg1)
 	    { OUTCHR(s, pad);
 	      w++;
@@ -2399,21 +2445,10 @@ Svfprintf(IOSTREAM *s, const char *fm, va_list args)
 		w--;
 	      }
 	    }
-	    while(*fs)
-	    { int c = next_chr((const char**)&fs, enc);
-	      OUTCHR(s, c);
-	    }
+	    OUTFS();
 	  }
 	} else
-	{ if ( fs == fbuf )		/* unaligned field, just output */
-	  { while(fs < fe)
-	      OUTCHR(s, *fs++);
-	  } else
-	  { while(*fs)
-	    { int c = next_chr((const char**)&fs, enc);
-	      OUTCHR(s, c);
-	    }
-	  }
+	{ OUTFS();
 	}
 	fm++;
 	if ( fs_malloced )
@@ -3334,6 +3369,36 @@ Sopen_file(const char *path, const char *how)
 }
 
 
+IOSTREAM *
+Sopen_iri_or_file(const char *path, const char *how)
+{ int sl;
+  IOSTREAM *s;
+
+  if ( (sl=file_name_is_iri(path)) )
+  { atom_t mname = how[0] == 'r' ? ATOM_read : ATOM_write;
+    const char *m;
+
+    if ( !iri_hook(path, IRI_OPEN, mname, 0, &s) )
+      return NULL;
+
+    for(m=&how[1]; *m; m++)
+    { if ( *m == 'b' )
+      { clear(s, SIO_TEXT);
+	s->encoding = ENC_OCTET;
+	s->newline  = SIO_NL_POSIX;
+      } else if ( *m == 'r' )
+      { clear(s, SIO_RECORDPOS);
+	s->position = NULL;
+      }
+    }
+  } else
+  { s = Sopen_file(path, how);
+  }
+
+  return s;
+}
+
+
 IOSTREAM *
 Sfdopen(int fd, const char *type)
 { intptr_t lfd;
