@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1995-2019, University of Amsterdam
+    Copyright (c)  1995-2022, University of Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -332,7 +333,7 @@ setOSPrologFlags(void)
 char *
 findExecutable(const char *module, char *exe, size_t exelen)
 { int n;
-  wchar_t wbuf[MAXPATHLEN];
+  wchar_t wbuf[PATH_MAX];
   HMODULE hmod;
 
   if ( module )
@@ -347,7 +348,7 @@ findExecutable(const char *module, char *exe, size_t exelen)
   } else
     hmod = NULL;
 
-  if ( (n = GetModuleFileNameW(hmod, wbuf, MAXPATHLEN)) > 0 )
+  if ( (n = GetModuleFileNameW(hmod, wbuf, PATH_MAX)) > 0 )
   { wbuf[n] = EOS;
     return _xos_long_file_name_toA(wbuf, exe, exelen);
   } else if ( module )
@@ -599,7 +600,7 @@ PRED_IMPL("win_shell", 3, win_shell3, 0)
 
 foreign_t
 pl_win_module_file(term_t module, term_t file)
-{ char buf[MAXPATHLEN];
+{ char buf[PATH_MAX];
   char *m;
   char *f;
 
@@ -825,13 +826,13 @@ PRED_IMPL("win_process_modules", 1, win_process_modules, 0)
 	int i;
 
 	for(i=0; i<lpcbNeeded/sizeof(HMODULE); i++)
-	{ wchar_t name[MAXPATHLEN];
+	{ wchar_t name[PATH_MAX];
 	  int n;
 
-	  if ( (n=GetModuleFileNameW(found[i], name, MAXPATHLEN)) > 0 )
+	  if ( (n=GetModuleFileNameW(found[i], name, PATH_MAX)) > 0 )
 	  { name[n] = EOS;
-	    char name_utf8[MAXPATHLEN*2];
-	    char pname[MAXPATHLEN*2];
+	    char name_utf8[PATH_MAX*2];
+	    char pname[PATH_MAX*2];
 
 	    if ( _xos_canonical_filenameW(name, name_utf8, sizeof(name_utf8), XOS_DOWNCASE) &&
 		 PrologPath(name_utf8, pname, sizeof(pname)) )
@@ -960,7 +961,7 @@ static const folderid folderids[] =
 
 static int
 unify_csidl_path(term_t t, int csidl)
-{ wchar_t buf[MAX_PATH];
+{ wchar_t buf[PATH_MAX];
 
   if ( SHGetSpecialFolderPathW(0, buf, csidl, FALSE) )
   { wchar_t *p;
@@ -1209,6 +1210,89 @@ PL_w32_running_under_wine(void)
 }
 
 
+		 /*******************************
+		 *	        MUI		*
+		 *******************************/
+
+
+static int
+langid_num(const wchar_t *s)
+{ int v=0;
+
+  for(; *s; s++)
+  { v =	v<<4;
+    if ( *s >= '0' && *s <= '9' )
+      v += *s-'0';
+    else if ( *s >= 'A' && *s <= 'F' )
+      v += *s-'A'+10;
+    else if ( *s >= 'a' && *s <= 'f' )
+      v += *s-'a'+10;
+    else
+      assert(0);
+  }
+
+  return v;
+}
+
+
+static
+PRED_IMPL("win_get_user_preferred_ui_languages", 2, win_get_user_preferred_ui_languages, 0)
+{ PRED_LD
+  char *how;
+  DWORD flags;
+  ULONG num, sz = 0;
+  wchar_t store[1024];
+  wchar_t *buf = store;
+  int rc = TRUE;
+
+  if ( PL_get_atom_chars(A1, &how) )
+  { if ( strcmp(how, "id") == 0 )
+      flags = MUI_LANGUAGE_ID;
+    else if ( strcmp(how, "name") == 0 )
+      flags = MUI_LANGUAGE_NAME;
+    else
+      return PL_domain_error("format", A1);
+  } else
+    return PL_type_error("atom", A1);
+
+  if ( (rc=GetUserPreferredUILanguages(flags, &num, NULL, &sz)) )
+  { if ( sz > sizeof(store)/sizeof(store[0]) )
+    { if ( !(buf = malloc(sz*sizeof(store[0]))) )
+	return PL_no_memory();
+    }
+    if ( (rc=GetUserPreferredUILanguages(flags, &num, buf, &sz)) )
+    { term_t tail = PL_copy_term_ref(A2);
+      term_t head = PL_new_term_ref();
+      wchar_t *s = buf;
+
+      while( rc && *s )
+      { if ( (rc=PL_unify_list(tail, head, tail)) )
+	{ if ( flags == MUI_LANGUAGE_NAME )
+	  { rc = PL_unify_wchars(head, PL_ATOM, (size_t)-1, s);
+	  } else
+	  { rc = PL_unify_integer(head, langid_num(s));
+	  }
+	}
+
+	if ( rc )
+	{ s += wcslen(s);
+	  s ++;
+	}
+      }
+      rc = rc && PL_unify_nil(tail);
+    }
+  }
+
+  if ( buf && buf != store )
+    free(buf);
+
+  return rc;
+}
+
+
+
+
+
 		 /*******************************
 		 *      PUBLISH PREDICATES	*
 		 *******************************/
@@ -1221,6 +1305,7 @@ BeginPredDefs(win)
   PRED_DEF("win_add_dll_directory",    2, win_add_dll_directory,    0)
   PRED_DEF("win_remove_dll_directory", 1, win_remove_dll_directory, 0)
   PRED_DEF("win_process_modules",      1, win_process_modules,	    0)
+  PRED_DEF("win_get_user_preferred_ui_languages", 2, win_get_user_preferred_ui_languages, 0)
 EndPredDefs
 
 #endif /*__WINDOWS__*/
